@@ -5,11 +5,12 @@
 */
 #include "windowthumbnail.h"
 // KF5
-#include <KWindowSystem>
+#include <KX11Extras>
 // Qt
 #include <QGuiApplication>
 #include <QIcon>
 #include <QOpenGLContext>
+#include <QOpenGLFunctions>
 #include <QQuickWindow>
 #include <QRunnable>
 #include <QSGImageNode>
@@ -26,6 +27,7 @@
 #include <GL/glx.h>
 typedef void (*glXBindTexImageEXT_func)(Display *dpy, GLXDrawable drawable, int buffer, const int *attrib_list);
 typedef void (*glXReleaseTexImageEXT_func)(Display *dpy, GLXDrawable drawable, int buffer);
+#include <fixx11h.h> // glx.h could include XLib.h
 #endif
 #if HAVE_EGL
 typedef EGLImageKHR (*eglCreateImageKHR_func)(EGLDisplay, EGLContext, EGLenum, EGLClientBuffer, const EGLint *);
@@ -121,7 +123,7 @@ void DiscardEglPixmapRunnable::run()
 
 QSGTexture *WindowTextureProvider::texture() const
 {
-    return m_texture.data();
+    return m_texture.get();
 }
 
 void WindowTextureProvider::setTexture(QSGTexture *texture)
@@ -226,13 +228,15 @@ void WindowThumbnail::itemChange(ItemChange change, const ItemChangeData &data)
     default:
         break;
     }
+
+    QQuickItem::itemChange(change, data);
 }
 
 void WindowThumbnail::releaseResources()
 {
     QQuickWindow::RenderStage m_renderStage = QQuickWindow::NoStage;
     if (m_textureProvider) {
-        window()->scheduleRenderJob(new DiscardTextureProviderRunnable(m_textureProvider), m_renderStage);
+        window()->scheduleRenderJob(new DiscardTextureProviderRunnable(m_textureProvider), QQuickWindow::AfterSynchronizingStage);
         m_textureProvider = nullptr;
     }
 
@@ -274,7 +278,7 @@ void WindowThumbnail::setWinId(uint32_t winId)
     if (m_winId == winId) {
         return;
     }
-    if (!KWindowSystem::self()->hasWId(winId)) {
+    if (!KX11Extras::self()->hasWId(winId)) {
         // invalid Id, don't updated
         return;
     }
@@ -382,8 +386,8 @@ bool WindowThumbnail::nativeEventFilter(const QByteArray &eventType, void *messa
 void WindowThumbnail::iconToTexture(WindowTextureProvider *textureProvider)
 {
     QIcon icon;
-    if (KWindowSystem::self()->hasWId(m_winId)) {
-        icon = KWindowSystem::self()->icon(m_winId, boundingRect().width(), boundingRect().height());
+    if (KX11Extras::self()->hasWId(m_winId)) {
+        icon = KX11Extras::self()->icon(m_winId, boundingRect().width(), boundingRect().height());
     } else {
         // fallback to plasma icon
         icon = QIcon::fromTheme(QStringLiteral("plasma"));
@@ -396,7 +400,12 @@ void WindowThumbnail::iconToTexture(WindowTextureProvider *textureProvider)
 #if HAVE_GLX
 bool WindowThumbnail::windowToTextureGLX(WindowTextureProvider *textureProvider)
 {
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     if (window()->openglContext()) {
+#else
+    const auto openglContext = static_cast<QOpenGLContext *>(window()->rendererInterface()->getResource(window(), QSGRendererInterface::OpenGLContextResource));
+    if (openglContext) {
+#endif
         if (!m_openGLFunctionsResolved) {
             resolveGLXFunctions();
         }
@@ -425,9 +434,16 @@ bool WindowThumbnail::windowToTextureGLX(WindowTextureProvider *textureProvider)
                 return false;
             }
 
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
             textureProvider->setTexture(window()->createTextureFromId(m_texture, QSize(geo->width, geo->height), QQuickWindow::TextureCanUseAtlas));
         }
         textureProvider->texture()->bind();
+#else
+            textureProvider->setTexture(
+                QNativeInterface::QSGOpenGLTexture::fromNative(m_texture, window(), QSize(geo->width, geo->height), QQuickWindow::TextureCanUseAtlas));
+        }
+        openglContext->functions()->glBindTexture(GL_TEXTURE_2D, m_texture);
+#endif
         bindGLXTexture();
         return true;
     }
@@ -455,11 +471,8 @@ bool WindowThumbnail::xcbWindowToTextureEGL(WindowTextureProvider *textureProvid
             auto geometryCookie = xcb_get_geometry_unchecked(c, m_pixmap);
 
             const EGLint attribs[] = {EGL_IMAGE_PRESERVED_KHR, EGL_TRUE, EGL_NONE};
-            m_image = ((eglCreateImageKHR_func)(m_eglCreateImageKHR))(eglGetCurrentDisplay(),
-                                                                      EGL_NO_CONTEXT,
-                                                                      EGL_NATIVE_PIXMAP_KHR,
-                                                                      (EGLClientBuffer)(uintptr_t)m_pixmap,
-                                                                      attribs);
+            m_image = ((eglCreateImageKHR_func)(
+                m_eglCreateImageKHR))(eglGetCurrentDisplay(), EGL_NO_CONTEXT, EGL_NATIVE_PIXMAP_KHR, (EGLClientBuffer)(uintptr_t)m_pixmap, attribs);
 
             if (m_image == EGL_NO_IMAGE_KHR) {
                 qDebug() << "failed to create egl image";
@@ -473,9 +486,16 @@ bool WindowThumbnail::xcbWindowToTextureEGL(WindowTextureProvider *textureProvid
                 size.setWidth(geo->width);
                 size.setHeight(geo->height);
             }
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
             textureProvider->setTexture(window()->createTextureFromId(m_texture, size, QQuickWindow::TextureCanUseAtlas));
         }
         textureProvider->texture()->bind();
+#else
+            textureProvider->setTexture(QNativeInterface::QSGOpenGLTexture::fromNative(m_texture, window(), size, QQuickWindow::TextureCanUseAtlas));
+        }
+        auto *openglContext = static_cast<QOpenGLContext *>(window()->rendererInterface()->getResource(window(), QSGRendererInterface::OpenGLContextResource));
+        openglContext->functions()->glBindTexture(GL_TEXTURE_2D, m_texture);
+#endif
         bindEGLTexture();
         return true;
     }
@@ -488,7 +508,11 @@ void WindowThumbnail::resolveEGLFunctions()
     if (display == EGL_NO_DISPLAY) {
         return;
     }
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     auto *context = window()->openglContext();
+#else
+    auto *context = static_cast<QOpenGLContext *>(window()->rendererInterface()->getResource(window(), QSGRendererInterface::OpenGLContextResource));
+#endif
     QList<QByteArray> extensions = QByteArray(eglQueryString(display, EGL_EXTENSIONS)).split(' ');
     if (extensions.contains(QByteArrayLiteral("EGL_KHR_image")) //
         || (extensions.contains(QByteArrayLiteral("EGL_KHR_image_base")) //
@@ -567,7 +591,11 @@ xcb_pixmap_t WindowThumbnail::pixmapForWindow()
 #if HAVE_GLX
 void WindowThumbnail::resolveGLXFunctions()
 {
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     auto *context = window()->openglContext();
+#else
+    auto *context = static_cast<QOpenGLContext *>(window()->rendererInterface()->getResource(window(), QSGRendererInterface::OpenGLContextResource));
+#endif
     QList<QByteArray> extensions = QByteArray(glXQueryExtensionsString(QX11Info::display(), QX11Info::appScreen())).split(' ');
     if (extensions.contains(QByteArrayLiteral("GLX_EXT_texture_from_pixmap"))) {
         m_bindTexImage = context->getProcAddress(QByteArrayLiteral("glXBindTexImageEXT"));
@@ -785,9 +813,7 @@ FbConfigInfo *getConfig(xcb_visualid_t visual)
         candidates.append(FBConfig{configs[i], depth, stencil, texture_format});
     }
 
-    if (count > 0) {
-        XFree(configs);
-    }
+    XFree(configs);
 
     std::stable_sort(candidates.begin(), candidates.end(), [](const FBConfig &left, const FBConfig &right) {
         if (left.depth < right.depth) {
@@ -975,3 +1001,5 @@ QSGTextureProvider *WindowThumbnail::textureProvider() const
 }
 
 } // namespace
+
+#include "moc_windowthumbnail.cpp"
