@@ -15,6 +15,7 @@
 #include <QPointer>
 #include <QQuickItem>
 #include <QSGSimpleTextureNode>
+#include <QSGTextureProvider>
 #include <QWindow>
 // xcb
 #if HAVE_XCB_COMPOSITE
@@ -32,7 +33,7 @@ class KWindowInfo;
 
 namespace Plasma
 {
-class WindowTextureNode;
+class WindowTextureProvider;
 
 /**
  * @brief Renders a thumbnail for the window specified by the @c winId property.
@@ -72,7 +73,11 @@ class WindowThumbnail : public QQuickItem, public QAbstractNativeEventFilter
 public:
     explicit WindowThumbnail(QQuickItem *parent = nullptr);
     ~WindowThumbnail() override;
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     bool nativeEventFilter(const QByteArray &eventType, void *message, long int *result) override;
+#else
+    bool nativeEventFilter(const QByteArray &eventType, void *message, qintptr *result) override;
+#endif
     QSGNode *updatePaintNode(QSGNode *oldNode, UpdatePaintNodeData *updatePaintNodeData) override;
 
     uint32_t winId() const;
@@ -82,6 +87,9 @@ public:
     qreal paintedHeight() const;
     bool thumbnailAvailable() const;
 
+    bool isTextureProvider() const override;
+    QSGTextureProvider *textureProvider() const override;
+
 Q_SIGNALS:
     void winIdChanged();
     void paintedSizeChanged();
@@ -92,8 +100,8 @@ protected:
     void releaseResources() override;
 
 private:
-    void iconToTexture(WindowTextureNode *textureNode);
-    void windowToTexture(WindowTextureNode *textureNode);
+    void iconToTexture(WindowTextureProvider *textureProvider);
+    void windowToTexture(WindowTextureProvider *textureProvider);
     bool startRedirecting();
     void stopRedirecting();
     void resetDamaged();
@@ -109,6 +117,7 @@ private:
     bool m_redirecting;
     bool m_damaged;
     int m_depth;
+    mutable WindowTextureProvider *m_textureProvider = nullptr;
 #if HAVE_XCB_COMPOSITE
     xcb_pixmap_t pixmapForWindow();
     bool m_openGLFunctionsResolved;
@@ -119,7 +128,7 @@ private:
     /*The following must *only* be used from the render thread*/
     uint m_texture;
 #if HAVE_GLX
-    bool windowToTextureGLX(WindowTextureNode *textureNode);
+    bool windowToTextureGLX(WindowTextureProvider *textureProvider);
     void resolveGLXFunctions();
     bool loadGLXTexture();
     void bindGLXTexture();
@@ -129,7 +138,7 @@ private:
     QFunctionPointer m_releaseTexImage;
 #endif // HAVE_GLX
 #if HAVE_EGL
-    bool xcbWindowToTextureEGL(WindowTextureNode *textureNode);
+    bool xcbWindowToTextureEGL(WindowTextureProvider *textureProvider);
     void resolveEGLFunctions();
     void bindEGLTexture();
     bool m_eglFunctionsResolved;
@@ -141,19 +150,16 @@ private:
 #endif
 };
 
-/**
- * @brief SimpleTextureNode which cleans up the texture
- *
- */
-class WindowTextureNode : public QSGSimpleTextureNode
+class WindowTextureProvider : public QSGTextureProvider
 {
+    Q_OBJECT
+
 public:
-    WindowTextureNode();
-    virtual ~WindowTextureNode();
-    void reset(QSGTexture *texture);
+    QSGTexture *texture() const override;
+    void setTexture(QSGTexture *texture);
 
 private:
-    QScopedPointer<QSGTexture> m_texture;
+    std::unique_ptr<QSGTexture> m_texture;
 };
 
 }
