@@ -7,13 +7,14 @@
 #ifndef QMENU_PROXY_H
 #define QMENU_PROXY_H
 
-#include <QObject>
-#include <QMenu>
-#include <QPointer>
-#include <QQmlListProperty>
-#include "qmenuitem.h"
 #include "enums.h"
 #include "plasma.h"
+#include "qmenuitem.h"
+#include <QMenu>
+#include <QObject>
+#include <QPointer>
+#include <QQmlListProperty>
+#include <QWindow>
 
 class QDeclarativeItem;
 
@@ -61,18 +62,23 @@ class QMenuProxy : public QObject
     Q_CLASSINFO("DefaultProperty", "content")
 
     /**
-     * This is a hint to the window manager that this window is a dialog or pop-up on behalf of the given window.
+     * This is a hint to the window manager that this window is a dialog or pop-up on
+     * behalf of the given window.
      */
     Q_PROPERTY(QWindow *transientParent READ transientParent WRITE setTransientParent NOTIFY transientParentChanged)
 
     /**
-     * the visualParent is used to position the menu. it can be an item on the scene, like a button (that will open the menu on clicked) or another menuitem (in this case this will be a submenu)
+     * The visualParent is used to position the menu. it can be an item on the
+     * scene, like a button (that will open the menu on clicked) or another menuitem
+     * (in this case this will be a submenu).
      */
     Q_PROPERTY(QObject *visualParent READ visualParent WRITE setVisualParent NOTIFY visualParentChanged())
     Q_PROPERTY(DialogStatus::Status status READ status NOTIFY statusChanged)
 
     /**
      * The default placement for the menu.
+     *
+     * (since 5.103) Will be automatically flipped horizontally in Right-to-left User Interfaces.
      */
     Q_PROPERTY(Plasma::Types::PopupPlacement placement READ placement WRITE setPlacement NOTIFY placementChanged)
 
@@ -181,5 +187,4 @@ private:
     Plasma::Types::PopupPlacement m_placement;
 };
 
-#endif //QMENU_PROXY_H
-
+#endif // QMENU_PROXY_H
