@@ -14,12 +14,31 @@
 #include <QUrl>
 
 #include <KConfigGroup>
+#include <plasma/plasma_export.h>
+#if PLASMA_ENABLE_DEPRECATED_SINCE(5, 94)
 #include <KPluginInfo>
+#endif
 
 #include <plasma/framesvg.h>
 #include <plasma/plasma.h>
 
+#if PLASMA_ENABLE_DEPRECATED_SINCE(5, 100)
 #include <KPackage/Package>
+#else
+namespace KPackage
+{
+class Package;
+}
+#endif
+namespace PlasmaQuick
+{
+class AppletQuickItem;
+class ConfigViewPrivate;
+class ConfigModelPrivate;
+class ConfigModel;
+};
+class DeclarativeAppletScript;
+#include <KPluginFactory>
 
 class KActionCollection;
 class KConfigLoader;
@@ -78,14 +97,31 @@ class PLASMA_EXPORT Applet : public QObject
 
 public:
     // CONSTRUCTORS
+
+    /**
+     * This constructor can be used with the KCoreAddons plugin loading system.
+     * The argument list is expected to have contain the KPackage of the applet,
+     * the meta data file path (for compatibility) and an applet ID which must be a base 10 number.
+     *
+     * @param parent a QObject parent; you probably want to pass in 0
+     * @param data, KPluginMetaData used to create this plugin
+     * @param args a list of strings containing the applet id
+     * @Since 5.86
+     */
+    Applet(QObject *parentObject, const KPluginMetaData &data, const QVariantList &args);
+
+#if PLASMA_ENABLE_DEPRECATED_SINCE(5, 86)
     /**
      * @param parent the QObject this applet is parented to
      * @param serviceId the name of the .desktop file containing the
      *      information about the widget
      * @param appletId a unique id used to differentiate between multiple
      *      instances of the same Applet type
+     * @deprecated Since 5.86, use Applet(QObject *, KPluginMetaData, QVariantList) instead
      */
+    PLASMA_DEPRECATED_VERSION(5, 86, "use Applet(QObject *, KPluginMetaData, QVariantList) instead")
     explicit Applet(QObject *parent = nullptr, const QString &serviceId = QString(), uint appletId = 0);
+#endif
 
 #if PLASMA_ENABLE_DEPRECATED_SINCE(5, 28)
     /**
@@ -101,14 +137,18 @@ public:
     explicit Applet(const KPluginInfo &info, QObject *parent = nullptr, uint appletId = 0);
 #endif
 
+#if PLASMA_ENABLE_DEPRECATED_SINCE(5, 86)
     /**
      * @param parent the QObject this applet is parented to
      * @param metadata the plugin information object for this Applet
      * @param appletId a unique id used to differentiate between multiple
      *      instances of the same Applet type
      * @since 5.27
+     * @deprecated Since 5.86, use Applet(QObject *, KPluginMetaData, QVariantList) instead
      */
+    PLASMA_DEPRECATED_VERSION(5, 86, "use Applet(QObject *, KPluginMetaData, QVariantList) instead")
     explicit Applet(const KPluginMetaData &metadata, QObject *parent = nullptr, uint appletId = 0);
+#endif
 
     ~Applet() override;
 
@@ -270,14 +310,17 @@ public:
     Package package() const;
 #endif
 
+#if PLASMA_ENABLE_DEPRECATED_SINCE(5, 100)
     /**
      * Accessor for the associated Package object if any.
      * Generally, only Plasmoids come in a Package.
      *
      * @return the Package object, or an invalid one if none
      * @since 5.6
+     * @deprecated Since 5.100, accessing an applets KPackage is deprecated. For using the metadata, use @p pluginMetaData instead.
      **/
     KPackage::Package kPackage() const;
+#endif
 
     /**
      * Called when any of the geometry constraints have been updated.
@@ -672,7 +715,7 @@ public Q_SLOTS:
     void runAssociatedApplication();
 
 protected:
-    // CONSTRUCTORS
+#if PLASMA_ENABLE_DEPRECATED_SINCE(5, 86)
     /**
      * This constructor is to be used with the plugin loading systems
      * found in KPluginInfo and KService. The argument list is expected
@@ -682,8 +725,11 @@ protected:
      * @param parent a QObject parent; you probably want to pass in 0
      * @param args a list of strings containing two entries: the service id
      *      and the applet id
+     * @deprecated Since 5.86, use Applet(QObject *, KPluginMetaData, QVariantList) instead
      */
+    PLASMA_DEPRECATED_VERSION(5, 86, "use Applet(QObject *, KPluginMetaData, QVariantList) instead")
     Applet(QObject *parent, const QVariantList &args);
+#endif
 
     // CONFIGURATION
     /**
@@ -749,6 +795,10 @@ protected:
     void timerEvent(QTimerEvent *event) override;
 
 private:
+    QString filePath(const QByteArray &key, const QString &filename = QString()) const;
+#if !PLASMA_ENABLE_DEPRECATED_SINCE(5, 100)
+    KPackage::Package kPackage() const;
+#endif
     /**
      * @internal This constructor is to be used with the Package loading system.
      *
@@ -779,19 +829,30 @@ private:
     friend class GraphicsViewAppletPrivate;
     friend class PluginLoader;
     friend class AssociatedApplicationManager;
+    friend class SvgPrivate;
+    friend class PlasmaQuick::AppletQuickItem;
+    friend class PlasmaQuick::ConfigModel;
+    friend class PlasmaQuick::ConfigModelPrivate;
+    friend class PlasmaQuick::ConfigViewPrivate;
+    friend DeclarativeAppletScript;
 };
 
 } // Plasma namespace
 
+#if PLASMA_ENABLE_DEPRECATED_SINCE(5, 88)
+
 /**
  * Register an applet when it is contained in a loadable module
+ * @deprecated Since 5.88, use K_PLUGIN_CLASS_WITH_JSON instead
  */
 /* clang-format off */
 #define K_EXPORT_PLASMA_APPLET(libname, classname) \
     K_PLUGIN_FACTORY(factory, registerPlugin<classname>();)
 
+/// @deprecated Since 5.88, use K_PLUGIN_CLASS_WITH_JSON instead
 #define K_EXPORT_PLASMA_APPLET_WITH_JSON(libname, classname, jsonFile) \
     K_PLUGIN_FACTORY_WITH_JSON(factory, jsonFile, registerPlugin<classname>();)
 /* clang-format on */
+#endif
 
 #endif // multiple inclusion guard
