@@ -11,9 +11,14 @@
 #include <QObject>
 #include <QStringList>
 
+#include <plasma/plasma_export.h>
+#if PLASMA_ENABLE_DEPRECATED_SINCE(5, 94)
 #include <KPluginInfo>
 #include <KService>
+#endif
 
+#include <KPluginFactory>
+#include <KPluginMetaData>
 #include <plasma/plasma.h>
 #include <plasma/service.h>
 
@@ -447,20 +452,24 @@ private:
     Q_PRIVATE_SLOT(d, void sourceDestroyed(QObject *object))
     Q_PRIVATE_SLOT(d, void scheduleSourcesUpdated())
 
-    DataEnginePrivate *const d;
+    DataEnginePrivate *d;
 };
 
 } // Plasma namespace
 
+#if PLASMA_ENABLE_DEPRECATED_SINCE(5, 88)
 /**
  * Register a data engine when it is contained in a loadable module
+ * @deprecated Since 5.88, use K_PLUGIN_CLASS_WITH_JSON instead
  */
 /* clang-format off */
 #define K_EXPORT_PLASMA_DATAENGINE(libname, classname) \
     K_PLUGIN_FACTORY(factory, registerPlugin<classname>();)
 
+/// @deprecated Since 5.88, use K_PLUGIN_CLASS_WITH_JSON instead
 #define K_EXPORT_PLASMA_DATAENGINE_WITH_JSON(libname, classname, jsonFile) \
     K_PLUGIN_FACTORY_WITH_JSON(factory, jsonFile, registerPlugin<classname>();)
 /* clang-format on */
+#endif
 
 #endif // multiple inclusion guard
