@@ -12,12 +12,10 @@
 #include <KActionCollection>
 
 #include "package.h"
-
-class KShortcutsDialog;
+#include <KPackage/Package>
 
 namespace Plasma
 {
-
 class Containment;
 
 class CoronaPrivate
