@@ -13,8 +13,8 @@
 
 #include <KConfigGroup>
 
-#include <plasma/plasma_export.h>
 #include <plasma/plasma.h>
+#include <plasma/plasma_export.h>
 
 class QIODevice;
 class QWidget;
@@ -23,7 +23,6 @@ class QQuickItem;
 
 namespace Plasma
 {
-
 class ServiceJob;
 class ServicePrivate;
 
@@ -87,7 +86,7 @@ public:
     /**
      * Destructor
      */
-    ~Service();
+    ~Service() override;
 
     /**
      * Sets the destination for this Service to operate on
@@ -168,8 +167,7 @@ protected:
      * @param parameters the parameters set by the user for the operation
      * @return a ServiceJob that can be started and monitored by the consumer
      */
-    virtual ServiceJob *createJob(const QString &operation,
-                                  QVariantMap &parameters) = 0;
+    virtual ServiceJob *createJob(const QString &operation, QVariantMap &parameters) = 0;
 
     /**
      * By default this is based on the file in plasma/services/name.operations, but can be
@@ -212,14 +210,11 @@ private:
 
 /**
  * Register a service when it is contained in a loadable module
+ * @deprecated Since 5.88, use K_PLUGIN_CLASS_WITH_JSON instead
  */
-#define K_EXPORT_PLASMA_SERVICE(libname, classname) \
-    K_PLUGIN_FACTORY(factory, registerPlugin<classname>();) \
-    K_EXPORT_PLUGIN_VERSION(PLASMA_VERSION)
+#define K_EXPORT_PLASMA_SERVICE(libname, classname) K_PLUGIN_FACTORY(factory, registerPlugin<classname>();)
 
-#define K_EXPORT_PLASMA_SERVICE_WITH_JSON(libname, classname, jsonFile) \
-    K_PLUGIN_FACTORY_WITH_JSON(factory, jsonFile, registerPlugin<classname>();) \
-    K_EXPORT_PLUGIN_VERSION(PLASMA_VERSION)
+/// @deprecated Since 5.88, use K_PLUGIN_CLASS_WITH_JSON instead
+#define K_EXPORT_PLASMA_SERVICE_WITH_JSON(libname, classname, jsonFile) K_PLUGIN_FACTORY_WITH_JSON(factory, jsonFile, registerPlugin<classname>();)
 
 #endif // multiple inclusion guard
-
